<?php

namespace Lc\ShopBundle\Form;

use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Lc\ShopBundle\Context\AddressInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Validator\Constraints\NotBlank;

class AddressType extends AbstractType
{
        protected $em;
        protected $security;

        public function __construct(EntityManagerInterface $entityManager, Security $security)
        {
                $this->em = $entityManager;
                $this->security = $security ;
        }

        public function buildForm(FormBuilderInterface $builder, array $options)
        {
                $builder
                        ->add('title', TextType::class, ['label' => 'Titre'])
                        ->add('type', ChoiceType::class, [
                                'label' => 'Type',
                                'choices' => [
                                        '--' => '',
                                        'Particulier' => 'individual',
                                        'Personne morale' => 'legal-person',
                                ]
                        ])
                        ->add('civility', ChoiceType::class, [
                                'label' => 'Civilité',
                                'required' => false,
                                'choices' => [
                                        '--' => '',
                                        'Femme' => 1,
                                        'Homme' => 0,
                                ],
                        ])
                        ->add('lastname', TextType::class, ['label' => 'Nom', 'required' => false])
                        ->add('firstname', TextType::class, ['label' => 'Prénom', 'required' => false])
                        ->add('address', TextareaType::class, ['label' => 'Adresse'])
                        ->add('zip', TextType::class, ['label' => 'Code postale'])
                        ->add('city', TextType::class, ['label' => 'Ville'])
                        ->add('country', TextType::class, ['label' => 'Pays'])
                        ->add('phone', TextType::class, ['label' => 'Téléphone', 'required' => false])
                        ->add('company', TextType::class, ['label' => 'Entreprise', 'required' => false])
                        ->add('siret', TextType::class, ['label' => 'SIRET', 'required' => false])
                        ->add('tva', TextType::class, ['label' => 'Numéro TVA', 'required' => false]);
        }

        public function configureOptions(OptionsResolver $resolver)
        {
                $resolver->setDefaults([
                        'label' => false,
                        'data_class' => $this->em->getClassMetadata(AddressInterface::class)->getName(),
                        'createdBy' => $this->security->getUser(),
                        'updatedBy' => $this->security->getUser(),
                ]);
        }
}
