<?php

namespace Lc\ShopBundle\Twig;

use App\Services\GlobalParam;
use Doctrine\ORM\EntityManagerInterface;
use Lc\ShopBundle\Form\Frontend\NewsletterType;
use Lc\ShopBundle\Repository\ProductCategoryRepository;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Security\Core\Security;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use Twig\TwigFunction;

class FrontendTwigExtension extends AbstractExtension
{
        protected $em ;
        protected $security ;
        protected $repositoryProductCategory ;
        protected $globalParam ;
        protected $formFactory ;

        public function __construct(EntityManagerInterface $em, Security $security, ProductCategoryRepository $repositoryProductCategory,
                GlobalParam $globalParam, FormFactoryInterface $formFactory)
        {
                $this->em = $em ;
                $this->security = $security ;
                $this->repositoryProductCategory = $repositoryProductCategory ;
                $this->globalParam = $globalParam ;
                $this->formFactory = $formFactory ;
        }

        public function getFunctions()
        {
                return array(
                        new TwigFunction('get_product_categories', [$this, 'getProductCategories']),
                        new TwigFunction('get_form_newsletter', [$this, 'getFormNewsletter']),
                );
        }

        public function getFilters()
        {
                return [
                        new TwigFilter('format_price', [$this, 'format_price']),
                ];
        }

        public function getProductCategories()
        {
                $categories = $this->repositoryProductCategory->findAllParents($this->globalParam->getCurrentMerchant()) ;
                return $categories ;
        }

        public function getFormNewsletter()
        {
                $form = $this->formFactory->create(NewsletterType::class);
                return $form->createView() ;
        }

        public function format_price($price)
        {
                $price = number_format($price, 2) ;
                $price = str_replace('.',',',$price) ;
                $price = $price .'&nbsp;€' ;
                return $price  ;
        }

}
